package de.intevation.flys.artifacts.model.minfo;

import org.apache.log4j.Logger;

import gnu.trove.TDoubleArrayList;
import de.intevation.flys.artifacts.model.NamedObjectImpl;

public class BedHeight
extends NamedObjectImpl
{
    private static Logger log = Logger.getLogger(BedHeight.class);

    protected TDoubleArrayList heights;
    protected TDoubleArrayList station;

    public BedHeight() {
        heights = new TDoubleArrayList();
        station = new TDoubleArrayList();
    }

    public BedHeight(String name) {
        super(name);
        heights = new TDoubleArrayList();
        station = new TDoubleArrayList();
    }

    public BedHeight(int capacity) {
        this(capacity, "");
    }

    public BedHeight(int capacity, String name) {
        super(name);
        heights = new TDoubleArrayList(capacity);
        station = new TDoubleArrayList(capacity);
    }

    public void add(double value, double station) {
        this.heights.add(value);
        this.station.add(station);
    }

    public int size() {
        return heights.size();
    }

    public double getHeight(int idx) {
        return heights.getQuick(idx);
    }

    public double [] getHeights() {
        return heights.toNativeArray();
    }

    public double [] get(int idx) {
        return get(idx, new double [3]);
    }

    public double [] get(int idx, double [] dst) {
        dst[0] = heights.getQuick(idx);
        dst[1] = station.getQuick(idx);
        return dst;
    }

   public double minHeights() {
        return heights.min();
    }

    public TDoubleArrayList getStations() {
        return this.station;
    }

    public double getHeight(double station) {
        if (this.station.indexOf(station) >= 0) {
            return this.heights.get(this.station.indexOf(station));
        }
        return Double.NaN;
    }


    public static void removeNaNs(TDoubleArrayList [] arrays) {

        int dest = 0;

        int A = arrays.length;
        int N = arrays[0].size();

        OUTER: for (int i = 0; i < N; ++i) {
            for (int j = 0; j < A; ++j) {
                TDoubleArrayList a = arrays[j];
                double v = a.getQuick(i);
                if (Double.isNaN(v)) {
                    continue OUTER;
                }
                a.setQuick(dest, v);
            }
            ++dest;
        }

        if (dest < N) {
            for (int i = 0; i < A; ++i) {
                arrays[i].remove(dest, N-dest);
            }
        }
    }

    public void removeNaNs() {
        removeNaNs(new TDoubleArrayList [] { heights });
    }
}
